/*
 * liballotc-node
 * Sangoma Transcoder Node Library
 *
 * Copyright (C) 2010 Sangoma Technologies
 * Nenad Corbic <ncorbic@sangoma.com>
 * 
 *
 * 
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *
 *  Moises Silva <moy@sangoma.com>
 *
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "soapH.h"
#include "ns1.nsmap"
#include "allotc_node.h"

allotc_log_func_t allotc_log_func;

static struct soap soap; // the gSOAP runtime context
allotc_init_cfg_t g_init;
static int detected_modules=0;
static allotc_mutex_t	allotc_node_lock;
static char allotc_server_url[255] = "http://127.10.10.1:64055";
int allotc_rtp_session_table_idx=0;
allotc_rtp_session_t allotc_rtp_session_table[MAX_ALLOTC_SESSIONS+1];

int __allotc_free_transcoding_session (struct allotc_codec_reply *codec_reply);

int allotc_find_rtp_session(struct allotc_codec_reply *codec_reply, allotc_rtp_session_t **rtp_session)
{
	int i;
	allotc_rtp_session_t *session;
	ALLOTC_DEBUG("%s\n",__FUNCTION__);
	// lock
	for (i=0;i<=MAX_ALLOTC_SESSIONS;i++) {
		session=&allotc_rtp_session_table[i];
		if (session && session->init != 0) {
			if (session->server_rtp_index == codec_reply->codec_rtp_session_idx) {
				ALLOTC_DEBUG("%s: Found rtp session 0x%08X init=%i\n", __FUNCTION__, codec_reply->codec_rtp_session_idx, session->init);
				*rtp_session=session;
				return 0;
			}
		}
	}
	
	// unlock
	return -1;
}

int allotc_release_rtp_session(allotc_rtp_session_t *session)
{
	ALLOTC_DEBUG("%s\n",__FUNCTION__);
	if (session->init == 2) {
		session->init++;
		return 0;

	} else if (session->init == 1) {
		session->init--;
		return 0;
	} else if (session->init == 3) {
		ALLOTC_DEBUG("Done with rtp session 0x%08X\n", session->server_rtp_index);
		session->init = 0;
		return 0;
	} else {
		ALLOTC_ERROR("%s: Error invalid session init value %i\n",
				__FUNCTION__,session->init);
		return -1;
	}

	return -1;
}

int allotc_get_free_rtp_session(allotc_rtp_session_t **rtp_session)
{
	int i;
	allotc_rtp_session_t *session;
	
	ALLOTC_DEBUG("%s\n",__FUNCTION__);

	// lock
	for (i=0;i<=MAX_ALLOTC_SESSIONS;i++) {
		allotc_rtp_session_table_idx++;
		if (allotc_rtp_session_table_idx > MAX_ALLOTC_SESSIONS) {
			allotc_rtp_session_table_idx=1;
		}

		session=&allotc_rtp_session_table[allotc_rtp_session_table_idx];
		if (session && session->init == 0) {
			session->init=1;
			session->index=allotc_rtp_session_table_idx;
			*rtp_session=session;
			return 0;
		}
	}
	
	// unlock
	return -1;
}

int allotc_get_existing_rtp_session(struct allotc_codec_request *codec_req,  allotc_rtp_session_t **rtp_session)
{
	int i;
	allotc_rtp_session_t *session;
	
	ALLOTC_DEBUG("%s\n",__FUNCTION__);

	// lock
	for (i=0;i<=MAX_ALLOTC_SESSIONS;i++) {

		session = &allotc_rtp_session_table[i];
		if (session && session->init == 1) {

			if (codec_req->a.codec_id == session->codec_req.b.codec_id &&
				codec_req->b.codec_id == session->codec_req.a.codec_id &&
				codec_req->a.ms == session->codec_req.b.ms &&
				codec_req->b.ms == session->codec_req.a.ms) {
	
				ALLOTC_DEBUG("GOT EXISING SESSION !\n");
	
				session->init++;
				*rtp_session=session;
				return 0;
			}
		}
	}
	
	// unlock
	return -1;
}

int allotc_detect_init_modules(allotc_init_cfg_t *cfg, int *detected)
{
	int err;

	soap_init2(&soap, SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
	soap.max_keep_alive = -1; // at most 100 calls per keep-alive session
	
	memcpy(&g_init,cfg,sizeof(g_init));
	allotc_log_func = g_init.log;
	detected_modules = *detected = 1;

	allotc_mutex_init(&allotc_node_lock,NULL);

	err = allotc_verify_init_cfg(&g_init);
	if (err) {
		return err;
	}


	return 0;
}

int allotc_activate_modules(allotc_init_cfg_t *cfg, int *activated)
{
	*activated=1;
	return 0;	
}
		
int allotc_detected_vocallo_modules(void)
{
	return 	detected_modules;	
}

		
int allotc_deactivate_modules(void)
{
	soap_destroy(&soap); // delete deserialized class instances (for C++)
	soap_end(&soap); // remove deserialized data and clean ups
	soap_done(&soap); // detach the gSOAP context

	allotc_mutex_destroy(&allotc_node_lock);
	return 0;
}

int allotc_create_transcoding_session (struct allotc_codec_request *u_codec_req, 
			  					      struct allotc_codec_reply *u_codec_reply,
									  int start_module)
{
	struct ns1__allotc_codec_request ns_codec_req;
	struct ns1__allotc_codec_reply ns_codec_reply;
	int err=-1;
	allotc_rtp_session_t *session = NULL;

	allotc_lock(&allotc_node_lock);

	if (sizeof(ns_codec_reply) != sizeof(*u_codec_reply)) {
		ALLOTC_ERROR("%s: Internal error ns_codec_reply=%i != codec_reply=%i\n",
					__FUNCTION__,sizeof(ns_codec_reply),sizeof(*u_codec_reply));
		goto allotc_create_transcoding_session_exit;
	}
	
	if (sizeof(ns_codec_req) != sizeof(*u_codec_req)) {
		ALLOTC_ERROR("%s: Internal error ns_codec_req=%i != codec_req=%i\n",
					__FUNCTION__,sizeof(ns_codec_req),sizeof(*u_codec_req));
		goto allotc_create_transcoding_session_exit;
	}

	

	err = allotc_get_existing_rtp_session(u_codec_req, &session);
	if (err) {
		err = allotc_get_free_rtp_session(&session);
		if (err) {
			goto allotc_create_transcoding_session_exit;
		}
	} else {
		/* We have determined that we already have an existing session we can reuse */
		memcpy(u_codec_reply, &session->codec_reply, sizeof(session->codec_reply));
		memcpy(&u_codec_reply->a, &session->codec_reply.b, sizeof(session->codec_reply.a));
		memcpy(&u_codec_reply->b, &session->codec_reply.a, sizeof(session->codec_reply.b));
	
		u_codec_reply->tx_fd = session->codec_reply.rx_fd;
		u_codec_reply->rx_fd = session->codec_reply.tx_fd;
		
		goto allotc_create_transcoding_session_exit;
	}

	memcpy(&session->codec_req, u_codec_req, sizeof(*u_codec_req));

	/* From this point on we are requesting a new session */

	err = g_init.create_rtp_port(session->codec_req.usr_priv,
						       session->codec_req.a.host_ip,
						       &session->codec_req.a.host_udp_port,
						       &session->codec_reply.tx_fd);
	if (err) {
		goto allotc_create_transcoding_session_exit;
	}
	session->codec_reply.a.host_udp_port	= session->codec_req.a.host_udp_port;

	err = g_init.create_rtp_port(session->codec_req.usr_priv,
						       session->codec_req.b.host_ip,
						       &session->codec_req.b.host_udp_port,
						       &session->codec_reply.rx_fd);
	if (err) {
		goto allotc_create_transcoding_session_exit;
	}
	session->codec_reply.b.host_udp_port	= session->codec_req.b.host_udp_port;


	memcpy(&ns_codec_req, &session->codec_req, sizeof(session->codec_req));
	memcpy(&ns_codec_reply, &session->codec_reply, sizeof(session->codec_reply));
	
	
	err = soap_call_ns1__allotc_create_transcoding_session(&soap,
												allotc_server_url,
												NULL,
												&ns_codec_req,
												&ns_codec_reply);
	
	if (err == SOAP_OK && ns_codec_reply.result == ALLOTC_OK) {

		ALLOTC_DEBUG("%s: Created session RTP ID = 0x%08X\n", __FUNCTION__, ns_codec_reply.codec_rtp_session_idx);
		memcpy(&session->codec_reply, &ns_codec_reply, sizeof(session->codec_reply));
		session->server_rtp_index = ns_codec_reply.codec_rtp_session_idx;
		session->server_mod_index = ns_codec_reply.codec_module_session_idx;

		err = g_init.create_rtp(session->codec_req.usr_priv,
						       &session->codec_req.a,
						       &session->codec_reply.a,
						       &session->codec_reply.tx_fd);
		if (err) {
			__allotc_free_transcoding_session(&session->codec_reply);
			goto allotc_create_transcoding_session_exit;
		}


		err = g_init.create_rtp(session->codec_req.usr_priv,
						       &session->codec_req.b,
						       &session->codec_reply.b,
						       &session->codec_reply.rx_fd);
		if (err) {
			__allotc_free_transcoding_session(&session->codec_reply);
			goto allotc_create_transcoding_session_exit;
		}

		memcpy(u_codec_req, &session->codec_req, sizeof(*u_codec_req));
		memcpy(u_codec_reply, &session->codec_reply, sizeof(*u_codec_reply));
		err = 0;
		

	} else { // an error occurred
		ALLOTC_ERROR("Create Transcoding Session Error\n");
		if (err != SOAP_OK) {
			soap_print_fault(&soap, stderr); // display the SOAP fault message on the stderr stream
			err = -1;
		} else {
			err = ns_codec_reply.result;
		}
		
		goto allotc_create_transcoding_session_exit;
	}


allotc_create_transcoding_session_exit:

	if (err && session) {

		if (session->codec_reply.a.host_udp_port) {
			g_init.release_rtp_port(session->codec_req.usr_priv,
					       session->codec_req.a.host_ip,
					       session->codec_req.a.host_udp_port,
					       session->codec_reply.tx_fd);
		}

		if (session->codec_reply.b.host_udp_port) {
			g_init.release_rtp_port(session->codec_req.usr_priv,
					       session->codec_req.b.host_ip,
					       session->codec_req.b.host_udp_port,
					       session->codec_reply.rx_fd);
		}

		allotc_release_rtp_session(session);
	}
	
	/* we must do this or we'll have leaks */		
	soap_destroy(&soap); // delete deserialized class instances (for C++)
	soap_end(&soap); // remove deserialized data and clean ups

	allotc_unlock(&allotc_node_lock);

	return err;
}

int __allotc_free_transcoding_session (struct allotc_codec_reply *codec_reply)
{
	struct ns1__allotc_codec_reply ns_codec_reply;
	struct ns1__allotc_codec_reply ns_codec_reply1;
	allotc_rtp_session_t *session=NULL;
	int result=-1;
	int err;

	if (sizeof(ns_codec_reply) != sizeof(*codec_reply)) {
		ALLOTC_ERROR("%s: Internal error ns_codec_reply=%i != codec_reply=%i\n",
					__FUNCTION__,sizeof(ns_codec_reply),sizeof(*codec_reply));
		return -1;
	}

	err=allotc_find_rtp_session(codec_reply,&session);
	if (err) {
		return err;
	}

	if (session->init == 1 || session->init == 3) {

		memcpy(&ns_codec_reply, codec_reply, sizeof(*codec_reply));
		memcpy(&ns_codec_reply1, codec_reply, sizeof(*codec_reply));
		
		ALLOTC_DEBUG("%s: Freeing session RTP ID = 0x%08X\n", __FUNCTION__, ns_codec_reply.codec_rtp_session_idx);

		err=soap_call_ns1__allotc_free_transcoding_session(&soap,
				allotc_server_url,
				NULL,
				&ns_codec_reply,&result);
		
		if (err == SOAP_OK && ns_codec_reply.result == ALLOTC_OK) {
			ALLOTC_DEBUG("%s: Freed session RTP ID = 0x%08X err:%d: result:%d:%d:OK\n", __FUNCTION__, ns_codec_reply.codec_rtp_session_idx, err, ns_codec_reply.result, result );

			err=result;

			g_init.destroy_rtp(session->codec_req.usr_priv, session->codec_reply.tx_fd);
			g_init.destroy_rtp(session->codec_req.usr_priv, session->codec_reply.rx_fd);

		} else {
			ALLOTC_ERROR("%s: Error freeing session RTP ID = 0x%08X Soap=%i Result=%i\n", __FUNCTION__, ns_codec_reply.codec_rtp_session_idx, err, ns_codec_reply.result);
			if (err != SOAP_OK) {
				soap_print_fault(&soap, stderr);
				err=-1;
			} else {
				err=ns_codec_reply.result;
			}
		}
	}

	allotc_release_rtp_session(session);

	soap_destroy(&soap);
	soap_end(&soap);
	
	return err;	

}

int allotc_free_transcoding_session (struct allotc_codec_reply *codec_reply)
{
	int err;

	allotc_lock(&allotc_node_lock);
	err=__allotc_free_transcoding_session(codec_reply);
	allotc_unlock(&allotc_node_lock);
	
	return err;	
}

int allotc_set_soap_server_url(const char *url)
{
	allotc_lock(&allotc_node_lock);
	strncpy(allotc_server_url, url, sizeof(allotc_server_url));
	allotc_server_url[sizeof(allotc_server_url)-1] = '\0';
	allotc_unlock(&allotc_node_lock);
	return 0;
}

int allotc_get_soap_server_url(char *url, size_t len)
{
	allotc_lock(&allotc_node_lock);
	strncpy(url, allotc_server_url, len);
	url[len] = '\0';
	allotc_unlock(&allotc_node_lock);
	return 0;
}

